@@ -1,5 +1,5 @@
 /*
- * Copyright © 2009-2012 Siyan Panayotov <xsisqox@gmail.com>
+ * Copyright © 2009-2014 Siyan Panayotov <siyan.panayotov@gmail.com>
  *
  * This file is part of Viewnior.
  *
@@ -162,6 +162,7 @@ vnr_prefs_set_default(VnrPrefs *prefs)
     prefs->jpeg_quality = 90;
     prefs->png_compression = 9;
     prefs->reload_on_save = FALSE;
+    prefs->show_menu_bar = FALSE;
     prefs->show_toolbar = TRUE;
     prefs->start_maximized = FALSE;
     prefs->start_slideshow = FALSE;
@@ -276,6 +277,7 @@ build_dialog (VnrPrefs *prefs)
     gtk_combo_box_append_text(desktop_env, "XFCE");
     gtk_combo_box_append_text(desktop_env, "LXDE");
     gtk_combo_box_append_text(desktop_env, "FluxBox");
+    gtk_combo_box_append_text(desktop_env, "Nitrogen");
     gtk_combo_box_set_active(desktop_env, prefs->desktop);
 
     gtk_box_pack_end (desktop_box, GTK_WIDGET(desktop_env), FALSE, FALSE, 0);
@@ -355,6 +357,7 @@ vnr_prefs_load (VnrPrefs *prefs)
     prefs->smooth_images = g_key_file_get_boolean (conf, "prefs", "smooth-images", &error);
     prefs->confirm_delete = g_key_file_get_boolean (conf, "prefs", "confirm-delete", &error);
     prefs->reload_on_save = g_key_file_get_boolean (conf, "prefs", "reload-on-save", &error);
+    prefs->show_menu_bar = g_key_file_get_boolean (conf, "prefs", "show-menu-bar", &error);
     prefs->show_toolbar = g_key_file_get_boolean (conf, "prefs", "show-toolbar", &error);
     prefs->start_maximized = g_key_file_get_boolean (conf, "prefs", "start-maximized", &error);
     prefs->slideshow_timeout = g_key_file_get_integer (conf, "prefs", "slideshow-timeout", &error);
@@ -443,6 +446,7 @@ vnr_prefs_save (VnrPrefs *prefs)
     g_key_file_set_boolean (conf, "prefs", "smooth-images", prefs->smooth_images);
     g_key_file_set_boolean (conf, "prefs", "confirm-delete", prefs->confirm_delete);
     g_key_file_set_boolean (conf, "prefs", "reload-on-save", prefs->reload_on_save);
+    g_key_file_set_boolean (conf, "prefs", "show-menu-bar", prefs->show_menu_bar);
     g_key_file_set_boolean (conf, "prefs", "show-toolbar", prefs->show_toolbar);
     g_key_file_set_boolean (conf, "prefs", "start-maximized", prefs->start_maximized);
     g_key_file_set_integer (conf, "prefs", "slideshow-timeout", prefs->slideshow_timeout);
@@ -489,3 +493,13 @@ vnr_prefs_set_show_toolbar (VnrPrefs *prefs, gboolean show_toolbar)
         vnr_prefs_save(prefs);
     }
 }
+
+void
+vnr_prefs_set_show_menu_bar (VnrPrefs *prefs, gboolean show_menu_bar)
+{
+    if(prefs->show_menu_bar != show_menu_bar)
+    {
+        prefs->show_menu_bar = show_menu_bar;
+        vnr_prefs_save(prefs);
+    }
+}
