@@ -1,5 +1,5 @@
 /*
- * Copyright © 2009-2012 Siyan Panayotov <xsisqox@gmail.com>
+ * Copyright © 2009-2014 Siyan Panayotov <siyan.panayotov@gmail.com>
  *
  * This file is part of Viewnior.
  *
@@ -35,6 +35,7 @@
 #include "vnr-message-area.h"
 #include "vnr-properties-dialog.h"
 #include "vnr-crop.h"
+#include "uni-exiv2.hpp"
 
 /* Timeout to hide the toolbar in fullscreen mode */
 #define FULLSCREEN_TIMEOUT 1000
@@ -58,7 +59,62 @@ static gboolean fullscreen_motion_cb(GtkWidget * widget, GdkEventMotion * ev, Vn
 static void open_with_launch_application_cb (GtkAction *action, VnrWindow *window);
 
 const gchar *ui_definition = "<ui>"
-  "<popup name=\"MainMenu\">"
+  "<menubar name=\"MainMenu\">"
+    "<menu action=\"File\">"
+      "<menuitem action=\"FileOpen\"/>"
+      "<menuitem action=\"FileOpenDir\"/>"
+      "<menu action=\"FileOpenWith\">"
+         "<placeholder name=\"AppEntries\"/>"
+      "</menu>"
+      "<separator/>"
+      "<menuitem action=\"FileSave\"/>"
+      "<menuitem action=\"FileReload\"/>"
+      "<separator/>"
+      "<menuitem action=\"FileProperties\"/>"
+      "<separator/>"
+      "<menuitem action=\"FileClose\"/>"
+    "</menu>"
+    "<menu action=\"Edit\">"
+      "<menuitem action=\"FileDelete\"/>"
+      "<separator/>"
+      "<menuitem action=\"EditPreferences\"/>"
+    "</menu>"
+    "<menu action=\"View\">"
+      "<menuitem action=\"ViewMenuBar\"/>"
+      "<menuitem action=\"ViewToolbar\"/>"
+      "<separator/>"
+      "<menuitem action=\"ViewZoomIn\"/>"
+      "<menuitem action=\"ViewZoomOut\"/>"
+      "<menuitem action=\"ViewZoomNormal\"/>"
+      "<menuitem action=\"ViewZoomFit\"/>"
+      "<separator/>"
+      "<menuitem name=\"Fullscreen\" action=\"ViewFullscreen\"/>"
+      "<menuitem name=\"Slideshow\" action=\"ViewSlideshow\"/>"
+      "<separator/>"
+      "<menuitem name=\"ResizeWindow\" action=\"ViewResizeWindow\"/>"
+    "</menu>"
+    "<menu action=\"Image\">"
+      "<menuitem action=\"ImageFlipVertical\"/>"
+      "<menuitem action=\"ImageFlipHorizontal\"/>"
+      "<separator/>"
+      "<menuitem action=\"ImageRotateCW\"/>"
+      "<menuitem action=\"ImageRotateCCW\"/>"
+      "<separator/>"
+      "<menuitem action=\"ImageCrop\"/>"
+      "<placeholder name=\"WallpaperEntry\"/>"
+    "</menu>"
+    "<menu action=\"Go\">"
+      "<menuitem name=\"GoPrevious\" action=\"GoPrevious\"/>"
+      "<menuitem name=\"GoNext\" action=\"GoNext\"/>"
+      "<separator/>"
+      "<menuitem name=\"GoFirst\" action=\"GoFirst\"/>"
+      "<menuitem name=\"GoLast\" action=\"GoLast\"/>"
+    "</menu>"
+    "<menu action=\"Help\">"
+      "<menuitem action=\"HelpAbout\"/>"
+    "</menu>"
+  "</menubar>"
+  "<popup name=\"ButtonMenu\">"
     "<menuitem action=\"FileOpen\"/>"
     "<menuitem action=\"FileOpenDir\"/>"
     "<menu action=\"FileOpenWith\">"
@@ -79,6 +135,7 @@ const gchar *ui_definition = "<ui>"
       "<menuitem action=\"ViewZoomNormal\"/>"
       "<menuitem action=\"ViewZoomFit\"/>"
       "<separator/>"
+      "<menuitem action=\"ViewMenuBar\"/>"
       "<menuitem action=\"ViewToolbar\"/>"
       "<menuitem name=\"Fullscreen\" action=\"ViewFullscreen\"/>"
       "<menuitem name=\"Slideshow\" action=\"ViewSlideshow\"/>"
@@ -128,6 +185,7 @@ const gchar *ui_definition = "<ui>"
     "<menuitem action=\"ViewZoomNormal\"/>"
     "<placeholder name=\"WallpaperEntry\"/>"
     "<separator/>"
+    "<menuitem name=\"MenuBar\" action=\"ViewMenuBar\"/>"
     "<menuitem name=\"Toolbar\" action=\"ViewToolbar\"/>"
     "<menuitem name=\"Fullscreen\" action=\"ViewFullscreen\"/>"
     "<separator/>"
@@ -141,7 +199,15 @@ const gchar *ui_definition = "<ui>"
 
 
 const gchar *ui_definition_wallpaper = "<ui>"
-  "<popup name=\"MainMenu\">"
+  "<menubar name=\"MainMenu\">"
+    "<menu action=\"Image\">"
+      "<placeholder name=\"WallpaperEntry\">"
+        "<separator/>"
+        "<menuitem name=\"Wallpaper\" action=\"SetAsWallpaper\"/>"
+      "</placeholder>"
+    "</menu>"
+  "</menubar>"
+  "<popup name=\"ButtonMenu\">"
     "<menu action=\"Image\">"
       "<placeholder name=\"WallpaperEntry\">"
         "<separator/>"
@@ -245,6 +311,13 @@ vnr_window_update_openwith_menu (VnrWindow *window)
         gtk_action_group_add_action (window->actions_open_with, action);
         g_object_unref (action);
 
+        gtk_ui_manager_add_ui (window->ui_mngr,
+                        window->open_with_menu_id,
+                        "/MainMenu/File/FileOpenWith/AppEntries",
+                        name,
+                        name,
+                        GTK_UI_MANAGER_MENUITEM,
+                        FALSE);
         gtk_ui_manager_add_ui (window->ui_mngr,
                         window->open_with_menu_id,
                         "/MainMenu/FileOpenWith/AppEntries",
@@ -462,6 +535,10 @@ vnr_window_fullscreen(VnrWindow *window)
 
     update_fs_filename_label(window);
     gtk_widget_hide (window->toolbar);
+
+    if (window->prefs->show_menu_bar)
+        gtk_widget_show (window->properties_button);
+
     gtk_widget_show (window->fs_controls);
 
     stop_slideshow(window);
@@ -512,6 +589,11 @@ vnr_window_unfullscreen(VnrWindow *window)
         uni_image_view_set_zoom_mode (UNI_IMAGE_VIEW(window->view),
                                       window->prefs->zoom);
 
+    if(window->prefs->show_menu_bar)
+        gtk_widget_hide (window->properties_button);
+    else
+        gtk_widget_hide (window->menu_bar);
+
     gtk_widget_hide (window->fs_controls);
 
     if(!window->prefs->show_toolbar)
@@ -683,7 +765,7 @@ rotate_pixbuf(VnrWindow *window, GdkPixbufRotation angle)
     else if(window->prefs->behavior_modify == VNR_PREFS_MODIFY_ASK)
         vnr_message_area_show_with_button(VNR_MESSAGE_AREA(window->msg_area),
                                           FALSE,
-                                          _("Save modifications?\nThis will overwrite the image and may reduce it's quality!"),
+                                          _("Save modifications?\nThis will overwrite the image and may reduce its quality!"),
                                           FALSE, GTK_STOCK_SAVE,
                                           G_CALLBACK(save_image_cb));
 }
@@ -741,7 +823,7 @@ flip_pixbuf(VnrWindow *window, gboolean horizontal)
     else if(window->prefs->behavior_modify == VNR_PREFS_MODIFY_ASK)
         vnr_message_area_show_with_button(VNR_MESSAGE_AREA(window->msg_area),
                                           FALSE,
-                                          _("Save modifications?\nThis will overwrite the image and may reduce it's quality!"),
+                                          _("Save modifications?\nThis will overwrite the image and may reduce its quality!"),
                                           FALSE, GTK_STOCK_SAVE,
                                           G_CALLBACK(save_image_cb));
 }
@@ -846,6 +928,9 @@ save_image_cb (GtkWidget *widget, VnrWindow *window)
     if(window->prefs->behavior_modify == VNR_PREFS_MODIFY_ASK)
         vnr_message_area_hide(VNR_MESSAGE_AREA(window->msg_area));
 
+    /* Store exiv2 metadata to cache, so we can restore it afterwards */
+    uni_read_exiv2_to_cache(VNR_FILE(window->file_list->data)->path);
+
     if(g_strcmp0(window->writable_format_name, "jpeg" ) == 0)
     {
         gchar *quality;
@@ -872,6 +957,7 @@ save_image_cb (GtkWidget *widget, VnrWindow *window)
                          VNR_FILE(window->file_list->data)->path,
                          window->writable_format_name, &error, NULL);
     }
+    uni_write_exiv2_from_cache(VNR_FILE(window->file_list->data)->path);
 
     if(!window->cursor_is_hidden)
         gdk_window_set_cursor(GTK_WIDGET(window)->window, gdk_cursor_new(GDK_LEFT_PTR));
@@ -904,7 +990,7 @@ static void
 vnr_window_main_menu_position (GtkMenu *menu, gint *x, gint *y, gboolean *push_in, gpointer user_data)
 {
 	VnrWindow *window = VNR_WINDOW(user_data);
-	GtkWidget *button = GTK_WIDGET(gtk_ui_manager_get_widget (window->ui_mngr, "/Toolbar/Properties"));
+    GtkWidget *button = window->properties_button;
 	GdkWindow *gdk_window = gtk_widget_get_window(button);
 	
 	gdk_window_get_position(gdk_window, x, y);
@@ -924,14 +1010,14 @@ vnr_window_cmd_open_menu (GtkToggleAction *action, VnrWindow *window)
 	if( !gtk_toggle_action_get_active (action)) {
 		return;
 	}
-	gtk_menu_popup(GTK_MENU(window->menu_bar), NULL, NULL, vnr_window_main_menu_position, window, 0, gtk_get_current_event_time());
+    gtk_menu_popup(GTK_MENU(window->button_menu), NULL, NULL, vnr_window_main_menu_position, window, 0, gtk_get_current_event_time());
 	return;
 }
 
 static void
 vnr_window_cmd_main_menu_hidden (GtkWidget *widget, gpointer user_data)
 {
-	gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(gtk_ui_manager_get_widget (VNR_WINDOW(user_data)->ui_mngr, "/Toolbar/Properties")), FALSE);
+    gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(VNR_WINDOW(user_data)->properties_button), FALSE);
 }
 
 static void
@@ -1179,6 +1265,9 @@ static void
 vnr_window_cmd_open(GtkAction *action, VnrWindow *window)
 {
     GtkWidget *dialog;
+    GtkFileFilter *img_filter = NULL;
+    GtkFileFilter *all_filter = NULL;
+
     dialog = gtk_file_chooser_dialog_new (_("Open Image"),
                           GTK_WINDOW(window),
                           GTK_FILE_CHOOSER_ACTION_OPEN,
@@ -1186,8 +1275,23 @@ vnr_window_cmd_open(GtkAction *action, VnrWindow *window)
                           GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
                           NULL);
 
+    img_filter = gtk_file_filter_new ();
+    g_assert (img_filter != NULL);
+    gtk_file_filter_add_pixbuf_formats (img_filter);
+    gtk_file_filter_add_mime_type (img_filter, "image/vnd.microsoft.icon");
+    gtk_file_filter_set_name (img_filter, _("All Images"));
+    gtk_file_chooser_add_filter (GTK_FILE_CHOOSER(dialog), img_filter);
+
+    all_filter = gtk_file_filter_new ();
+    g_assert(all_filter != NULL);
+    gtk_file_filter_add_pattern (all_filter, "*");
+    gtk_file_filter_set_name (all_filter, _("All Files"));
+    gtk_file_chooser_add_filter (GTK_FILE_CHOOSER(dialog), all_filter);
+
     gtk_window_set_modal (GTK_WINDOW(dialog), FALSE);
     gtk_file_chooser_set_select_multiple(GTK_FILE_CHOOSER(dialog), TRUE);
+    
+    gtk_file_chooser_set_filter (GTK_FILE_CHOOSER(dialog), img_filter);
 
     gchar *dirname;
     if(window->file_list != NULL)
@@ -1202,6 +1306,9 @@ vnr_window_cmd_open(GtkAction *action, VnrWindow *window)
                       window);
 
     gtk_widget_show_all (GTK_WIDGET(dialog));
+
+    /* This only works when here. */
+    gtk_file_chooser_set_show_hidden (GTK_FILE_CHOOSER(dialog), window->prefs->show_hidden);
 }
 
 static void
@@ -1231,6 +1338,9 @@ vnr_window_cmd_open_dir(GtkAction *action, VnrWindow *window)
                       window);
 
     gtk_widget_show_all (GTK_WIDGET(dialog));
+
+    /* This only works when here. */
+    gtk_file_chooser_set_show_hidden (GTK_FILE_CHOOSER(dialog), window->prefs->show_hidden);
 }
 
 static void
@@ -1238,7 +1348,7 @@ vnr_window_cmd_about (GtkAction *action, VnrWindow *window)
 {
     static const char *authors[] = {
         "Programming & icon design",
-        "\tSiyan Panayotov <xsisqox@gmail.com>",
+        "\tSiyan Panayotov <siyan.panayotov@gmail.com>",
         "\nRefer to source code from GtkImageView",
         NULL
     };
@@ -1258,13 +1368,13 @@ vnr_window_cmd_about (GtkAction *action, VnrWindow *window)
     gtk_show_about_dialog (GTK_WINDOW (window),
                    "program-name", "Viewnior",
                    "version", VERSION,
-                   "copyright", "Copyright \xc2\xa9 2009-2012 Siyan Panayotov <xsisqox@gmail.com>",
+                   "copyright", "Copyright \xc2\xa9 2009-2014 Siyan Panayotov <siyan.panayotov@gmail.com>",
                    "comments",_("Elegant Image Viewer"),
                    "authors", authors,
                    "logo-icon-name", "viewnior",
                    "wrap-license", TRUE,
                    "license", license,
-                   "website", "http://xsisqox.github.com/Viewnior",
+                   "website", "http://xsisqox.github.io/Viewnior",
                    "translator-credits", _( "translator-credits" ),
                    NULL);
 }
@@ -1316,6 +1426,12 @@ vnr_set_wallpaper(GtkAction *action, VnrWindow *win)
 						"-f", VNR_FILE(win->file_list->data)->path, 
 						NULL);
 				break;
+			case VNR_PREFS_DESKTOP_NITROGEN:
+				execlp("nitrogen", "nitrogen", 
+						"--set-zoom-fill",
+						VNR_FILE(win->file_list->data)->path, 
+						NULL);
+				break;
 			default:
 				_exit(0);	
 		}
@@ -1337,6 +1453,30 @@ vnr_window_cmd_fullscreen (GtkAction *action, VnrWindow *window)
         vnr_window_unfullscreen (window);
 }
 
+static void
+vnr_window_cmd_menu_bar (GtkAction *action, VnrWindow *window)
+{
+    gboolean show;
+
+    show = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+    vnr_prefs_set_show_menu_bar(window->prefs, show);
+
+    if(window->mode != VNR_WINDOW_MODE_NORMAL)
+       return;
+
+
+    if (show)
+    {
+        gtk_widget_show (window->menu_bar);
+        gtk_widget_hide (window->properties_button);
+    }
+    else
+    {
+        gtk_widget_hide (window->menu_bar);
+        gtk_widget_show (window->properties_button);
+    }
+}
+
 static void
 vnr_window_cmd_toolbar (GtkAction *action, VnrWindow *window)
 {
@@ -1570,7 +1710,7 @@ vnr_window_cmd_crop(GtkAction *action, VnrWindow *window)
     else if(window->prefs->behavior_modify == VNR_PREFS_MODIFY_ASK)
         vnr_message_area_show_with_button(VNR_MESSAGE_AREA(window->msg_area),
                                           FALSE,
-                                          _("Save modifications?\nThis will overwrite the image and may reduce it's quality!"),
+                                          _("Save modifications?\nThis will overwrite the image and may reduce its quality!"),
                                           FALSE, GTK_STOCK_SAVE,
                                           G_CALLBACK(save_image_cb));
 
@@ -1687,6 +1827,9 @@ static const GtkToggleActionEntry toggle_entries_image[] = {
 };
 
 static const GtkToggleActionEntry toggle_entries_window[] = {
+    { "ViewMenuBar", NULL, N_("Menu Bar"), NULL,
+      N_("Show Menu Bar"),
+      G_CALLBACK (vnr_window_cmd_menu_bar) },
     { "ViewToolbar", NULL, N_("Toolbar"), NULL,
       N_("Show Toolbar"),
       G_CALLBACK (vnr_window_cmd_toolbar) },
@@ -1955,19 +2098,19 @@ vnr_window_init (VnrWindow * window)
     gtk_ui_manager_insert_action_group (window->ui_mngr,
                                         window->actions_image, 0);
     /**********/
-    window->action_toolbar = gtk_action_group_new("MenuActionToolbar");
+    window->actions_bars = gtk_action_group_new("MenuActionsBars");
 
 
-    gtk_action_group_set_translation_domain (window->action_toolbar,
-                                             GETTEXT_PACKAGE);
+    gtk_action_group_set_translation_domain (window->actions_bars,
+                                              GETTEXT_PACKAGE);
 
-    gtk_action_group_add_toggle_actions (window->action_toolbar,
+    gtk_action_group_add_toggle_actions (window->actions_bars,
                                          toggle_entries_window,
                                          G_N_ELEMENTS (toggle_entries_window),
                                          window);
 
     gtk_ui_manager_insert_action_group (window->ui_mngr,
-                                        window->action_toolbar, 0);
+                                        window->actions_bars, 0);
 
     /*****************/
     window->actions_collection = gtk_action_group_new("MenuActionsCollection");
@@ -2020,7 +2163,7 @@ vnr_window_init (VnrWindow * window)
     gtk_action_group_set_sensitive(window->actions_image, FALSE);
     gtk_action_group_set_sensitive(window->actions_static_image, FALSE);
     gtk_action_group_set_sensitive(window->action_save, FALSE);
-    gtk_action_group_set_sensitive(window->action_toolbar, TRUE);
+    gtk_action_group_set_sensitive(window->actions_bars, TRUE);
 
     /* Continue with layout */
 
@@ -2033,7 +2176,14 @@ vnr_window_init (VnrWindow * window)
 
     window->menu_bar = gtk_ui_manager_get_widget (window->ui_mngr, "/MainMenu");
     g_assert(GTK_IS_WIDGET(window->menu_bar));
-    gtk_menu_attach_to_widget (GTK_MENU(window->menu_bar), GTK_WIDGET(gtk_ui_manager_get_widget (window->ui_mngr, "/Toolbar/Properties")), NULL);
+    gtk_box_pack_start (GTK_BOX (window->menus), window->menu_bar, FALSE,FALSE,0);
+
+    window->properties_button = gtk_ui_manager_get_widget (window->ui_mngr, "/Toolbar/Properties");
+    g_assert(GTK_IS_WIDGET(window->properties_button));
+
+    window->button_menu = gtk_ui_manager_get_widget (window->ui_mngr, "/ButtonMenu");
+    g_assert(GTK_IS_WIDGET(window->button_menu));
+    gtk_menu_attach_to_widget (GTK_MENU(window->button_menu), GTK_WIDGET(window->properties_button), NULL);
 
     window->toolbar = gtk_ui_manager_get_widget (window->ui_mngr, "/Toolbar");
     g_assert(GTK_IS_WIDGET(window->toolbar));
@@ -2051,8 +2201,17 @@ vnr_window_init (VnrWindow * window)
 
     gtk_widget_hide(get_fs_controls(window));
 
+    // Apply menu bar preference
+    action = gtk_action_group_get_action (window->actions_bars,
+                                          "ViewMenuBar");
+    if(!window->prefs->show_menu_bar)
+        gtk_widget_hide (window->menu_bar);
+    else
+        gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), TRUE);
+
+
     // Apply toolbar preference
-    action = gtk_action_group_get_action (window->action_toolbar,
+    action = gtk_action_group_get_action (window->actions_bars,
                                           "ViewToolbar");
     if(!window->prefs->show_toolbar)
         gtk_widget_hide (window->toolbar);
@@ -2105,7 +2264,7 @@ vnr_window_init (VnrWindow * window)
     g_signal_connect (G_OBJECT (window->view), "drag-data-get",
                       G_CALLBACK (window_drag_begin_cb), window);
                       
-	g_signal_connect (G_OBJECT (window->menu_bar), "hide", 
+	g_signal_connect (G_OBJECT (window->button_menu), "hide", 
 					  G_CALLBACK(vnr_window_cmd_main_menu_hidden), window);
 
     gtk_window_add_accel_group (GTK_WINDOW (window),
